@@ -1,3 +1,20 @@
+//This unit uses a subset of the functions from the nifti1_io available from
+//  https://sourceforge.net/projects/niftilib/files/nifticlib/
+//These functions were extended by Chris Rorden (2014) and maintain the same license
+/*****===================================================================*****/
+/*****     Sample functions to deal with NIFTI-1 and ANALYZE files       *****/
+/*****...................................................................*****/
+/*****            This code is released to the public domain.            *****/
+/*****...................................................................*****/
+/*****  Author: Robert W Cox, SSCC/DIRP/NIMH/NIH/DHHS/USA/EARTH          *****/
+/*****  Date:   August 2003                                              *****/
+/*****...................................................................*****/
+/*****  Neither the National Institutes of Health (NIH), nor any of its  *****/
+/*****  employees imply any warranty of usefulness of this software for  *****/
+/*****  any purpose, and do not assume any liability for damages,        *****/
+/*****  incidental or otherwise, caused by any use of this document.     *****/
+/*****===================================================================*****/
+
 #include "nifti1_io_core.h"
 #include <math.h>
 #include <stdlib.h>
@@ -12,6 +29,9 @@
 #include <unistd.h>
 #endif
 
+#include "print.h"
+
+#ifndef HAVE_R
 void nifti_swap_8bytes( size_t n , void *ar )    // 4 bytes at a time
 {
     size_t ii ;
@@ -58,6 +78,7 @@ void nifti_swap_2bytes( size_t n , void *ar )    // 2 bytes at a time
     }
     return ;
 }
+#endif
 
 int isSameFloat (float a, float b) {
     return (fabs (a - b) <= FLT_EPSILON);
@@ -65,19 +86,19 @@ int isSameFloat (float a, float b) {
 
 ivec3 setiVec3(int x, int y, int z)
 {
-    ivec3 v = {x, y, z};
+    ivec3 v = {{x, y, z}};
     return v;
 }
 
 vec3 setVec3(float x, float y, float z)
 {
-    vec3 v = {x, y, z};
+    vec3 v = {{x, y, z}};
     return v;
 }
 
 vec4 setVec4(float x, float y, float z)
 {
-    vec4 v= {x, y, z, 1};
+    vec4 v= {{x, y, z, 1}};
     return v;
 }
 
@@ -130,7 +151,7 @@ mat44 nifti_dicom2mat(float orient[7], float patientPosition[4], float xyzMM[4])
     mat33 Q, diagVox;
     Q.m[0][0] = orient[1]; Q.m[0][1] = orient[2] ; Q.m[0][2] = orient[3] ; // load Q
     Q.m[1][0] = orient[4]; Q.m[1][1] = orient[5] ; Q.m[1][2] = orient[6];
-    //printf("Orient %g %g %g %g %g %g\n",orient[1],orient[2],orient[3],orient[4],orient[5],orient[6] );
+    //printMessage("Orient %g %g %g %g %g %g\n",orient[1],orient[2],orient[3],orient[4],orient[5],orient[6] );
     /* normalize row 1 */
     double val = Q.m[0][0]*Q.m[0][0] + Q.m[0][1]*Q.m[0][1] + Q.m[0][2]*Q.m[0][2] ;
     if( val > 0.0l ){
@@ -167,6 +188,7 @@ mat44 nifti_dicom2mat(float orient[7], float patientPosition[4], float xyzMM[4])
     return Q44;
 }
 
+#ifndef HAVE_R
 float nifti_mat33_determ( mat33 R )   /* determinant of 3x3 matrix */
 {
     double r11,r12,r13,r21,r22,r23,r31,r32,r33 ;
@@ -189,6 +211,7 @@ mat33 nifti_mat33_mul( mat33 A , mat33 B )  /* multiply 2 3x3 matrices */
             + A.m[i][2] * B.m[2][j] ;
     return C ;
 }
+#endif
 
 mat44 nifti_mat44_mul( mat44 A , mat44 B )  /* multiply 2 3x3 matrices */
 {
@@ -212,6 +235,7 @@ mat33 nifti_mat33_transpose( mat33 A )  /* transpose 3x3 matrix */
     return B;
 }
 
+#ifndef HAVE_R
 mat33 nifti_mat33_inverse( mat33 R )   /* inverse of 3x3 matrix */
 {
     double r11,r12,r13,r21,r22,r23,r31,r32,r33 , deti ;
@@ -468,7 +492,7 @@ mat44 nifti_mat44_inverse( mat44 R )
     Q.m[3][3] = (deti == 0.0l) ? 0.0l : 1.0l ; // failure flag if deti == 0
     return Q ;
 }
-
+#endif
 
 
 
